example_headers = ["Inorg_arb_examplepy_plugin_0_fun_count_drp_0.02",
"_N?_examplepy_plugin_0_False_molarity_drp_0.02",
"Ox_N?_examplepy_plugin_0_True_molarity_drp_0.02",
"Org_arb_examplepy_plugin_0_fun_count_drp_0.02",
"_N?_examplepy_plugin_0_True_molarity_drp_0.02",
"_arb_examplepy_plugin_0_dull_molarity_drp_0.02",
"Ox_arb_examplepy_plugin_0_fun_molarity_drp_0.02",
"Org_N?_examplepy_plugin_0_False_molarity_drp_0.02",
"Solv_N?_examplepy_plugin_0_False_molarity_drp_0.02",
"Org_fs_examplepy_plugin_0_1_count_drp_0.02",
"Solv_fs_examplepy_plugin_0_1_count_drp_0.02",
"Solv_fs_examplepy_plugin_0_3_count_drp_0.02",
"Inorg_N?_examplepy_plugin_0_True_molarity_drp_0.02",
"Org_arb_examplepy_plugin_0_dull_count_drp_0.02",
"Org_N?_examplepy_plugin_0_True_molarity_drp_0.02",
"Org_arb_examplepy_plugin_0_dull_molarity_drp_0.02",
"Inorg_N?_examplepy_plugin_0_False_count_drp_0.02",
"_fs_examplepy_plugin_0_3_count_drp_0.02",
"Solv_arb_examplepy_plugin_0_fun_molarity_drp_0.02",
"Solv_fs_examplepy_plugin_0_2_molarity_drp_0.02",
"Solv_arb_examplepy_plugin_0_fun_count_drp_0.02",
"Ox_fs_examplepy_plugin_0_3_count_drp_0.02",
"Inorg_fs_examplepy_plugin_0_1_count_drp_0.02",
"Ox_arb_examplepy_plugin_0_fun_count_drp_0.02",
"_fs_examplepy_plugin_0_2_count_drp_0.02",
"Ox_fs_examplepy_plugin_0_1_molarity_drp_0.02",
"_N?_examplepy_plugin_0_True_count_drp_0.02",
"Ox_N?_examplepy_plugin_0_False_count_drp_0.02",
"Inorg_fs_examplepy_plugin_0_2_molarity_drp_0.02",
"Inorg_fs_examplepy_plugin_0_2_count_drp_0.02",
"Inorg_N?_examplepy_plugin_0_True_count_drp_0.02",
"_arb_examplepy_plugin_0_fun_molarity_drp_0.02",
"Solv_arb_examplepy_plugin_0_dull_molarity_drp_0.02",
"Inorg_arb_examplepy_plugin_0_dull_molarity_drp_0.02",
"Inorg_arb_examplepy_plugin_0_dull_count_drp_0.02",
"Ox_arb_examplepy_plugin_0_dull_count_drp_0.02",
"_arb_examplepy_plugin_0_fun_count_drp_0.02",
"Ox_fs_examplepy_plugin_0_1_count_drp_0.02",
"Ox_fs_examplepy_plugin_0_3_molarity_drp_0.02",
"Inorg_N?_examplepy_plugin_0_False_molarity_drp_0.02",
"Org_fs_examplepy_plugin_0_3_count_drp_0.02",
"Solv_arb_examplepy_plugin_0_dull_count_drp_0.02",
"Ox_fs_examplepy_plugin_0_2_molarity_drp_0.02",
"_fs_examplepy_plugin_0_1_count_drp_0.02",
"Org_arb_examplepy_plugin_0_fun_molarity_drp_0.02",
"Ox_fs_examplepy_plugin_0_2_count_drp_0.02",
"Org_fs_examplepy_plugin_0_1_molarity_drp_0.02",
"Solv_fs_examplepy_plugin_0_2_count_drp_0.02",
"_fs_examplepy_plugin_0_3_molarity_drp_0.02",
"_fs_examplepy_plugin_0_1_molarity_drp_0.02",
"Ox_arb_examplepy_plugin_0_dull_molarity_drp_0.02",
"Org_N?_examplepy_plugin_0_True_count_drp_0.02",
"Solv_N?_examplepy_plugin_0_False_count_drp_0.02",
"Org_fs_examplepy_plugin_0_3_molarity_drp_0.02",
"_fs_examplepy_plugin_0_2_molarity_drp_0.02",
"Inorg_arb_examplepy_plugin_0_fun_molarity_drp_0.02",
"Inorg_fs_examplepy_plugin_0_3_molarity_drp_0.02",
"Solv_fs_examplepy_plugin_0_1_molarity_drp_0.02",
"Ox_N?_examplepy_plugin_0_False_molarity_drp_0.02",
"Solv_N?_examplepy_plugin_0_True_molarity_drp_0.02",
"Solv_N?_examplepy_plugin_0_True_count_drp_0.02",
"Ox_N?_examplepy_plugin_0_True_count_drp_0.02",
"_N?_examplepy_plugin_0_False_count_drp_0.02",
"Org_fs_examplepy_plugin_0_2_molarity_drp_0.02",
"Org_fs_examplepy_plugin_0_2_count_drp_0.02",
"_arb_examplepy_plugin_0_dull_count_drp_0.02",
"Inorg_fs_examplepy_plugin_0_3_count_drp_0.02",
"Org_N?_examplepy_plugin_0_False_count_drp_0.02",
"Inorg_fs_examplepy_plugin_0_1_molarity_drp_0.02",
"Solv_fs_examplepy_plugin_0_3_molarity_drp_0.02",
]


